/*
 * kwin_iaora.cpp - IaOra window decoration for KDE
 *                  based on the Skulpture window decoration
 *
 * Copyright 2008 Gustavo Pichorim Boiko <boiko@mandriva.com>
 * Skulpture Copyright 2008 Christoph Feck
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "kwin_iaora.h"
#include <KDE/KLocale>
#include <KDE/KConfig>
#include <KDE/KConfigGroup>
#include <QtGui/QStylePainter>
#include <QtGui/QStyle>
#include <QtGui/QStyleFactory>
#include <QtGui/QStyleOption>
#include <QtGui/QBitmap>
#include <QtCore/QLibrary>
#include <QtCore/QPluginLoader>
#include <QtGui/QStylePlugin>

#include <QtCore/QCoreApplication>
#include <QtCore/QDebug>
#include <cstdio>


/*-----------------------------------------------------------------------*/
/**
 * The IaOra window decoration for KDE actually is a wrapper.
 * It uses Qt's MDI window decoration. As such, it is very lightweight.
 *
 */

extern "C" KDE_EXPORT KDecorationFactory *create_factory()
{
	return new IaOraDecorationFactory();
}


/*-----------------------------------------------------------------------*/
/*
 * When KWin is already running, it doesn't notice that
 * a new style has been installed, and QStyleFactory will
 * not find the new style.
 *
 * This function works around this Qt problem by manually
 * loading the style.
 *
 */

static void setIaOraStyle(QWidget *widget)
{
	static QStyle *iaoraStyle = 0;
	QStyle *style = 0;
//	QStyle *style = QStyleFactory::create(QString::fromUtf8("iaora"));
	if (style) {
//		qDebug() << "created style" << style->objectName();
	} else {
//		printf("no style\n");
	}
	if (!style && iaoraStyle) {
		style = iaoraStyle;
	}
	if (!style) {
		bool loaded = false;
		QStringList paths = QCoreApplication::libraryPaths();
		QLibrary library;
		foreach (QString path, paths) {
			library.setFileName(path + QString::fromUtf8("/styles/iaora-qt"));
			if (library.load()) {
				loaded = true;
				break;
			}
		}
		if (loaded) {
//			printf("loaded\n");
		} else {
//			printf("NOT loaded\n");
		}
		if (loaded) {
			QString libName = library.fileName();
			library.unload();
		//	qDebug() << QCoreApplication::libraryPaths();
			// find system specific extensions
			// try loading it ourself...
//			qDebug() << "loading" << library.fileName();
			QPluginLoader plugin(libName);
			QObject *instance = plugin.instance();
			if (instance) {
//				printf("found instance\n");
				QStyleFactoryInterface *factory = qobject_cast<QStyleFactoryInterface*>(instance);
				if (factory) {
//					printf("found factory\n");
					style = factory->create(QString::fromUtf8("iaora-qt"));
					if (style) {
//						printf("created style\n");
						iaoraStyle = style;
					}
				}
			}
		}
	}
	if (style) {
		widget->setStyle(style);
	} else {
//		printf("no iaora style loaded, restart kwin\n");
	}
}


/*-----------------------------------------------------------------------*/

IaOraDecorationButton::IaOraDecorationButton(ButtonType type, KCommonDecoration *parent)
	: KCommonDecorationButton(type, parent)
{
	init();
}


void IaOraDecorationButton::init()
{
	setAttribute(Qt::WA_PaintOnScreen, false);
	setAttribute(Qt::WA_NoSystemBackground, true);
	setAutoFillBackground(false);
	setFocusPolicy(Qt::NoFocus);
	setAttribute(Qt::WA_OpaquePaintEvent, false);
	setAttribute(Qt::WA_Hover, true);
	setIaOraStyle(this);
}


IaOraDecorationButton::~IaOraDecorationButton()
{
	/* */
}


void IaOraDecorationButton::reset(unsigned long /*changed*/)
{
	/* NOTE: must be implemented, because it is declared pure */
}


void IaOraDecorationButton::initStyleOption(QStyleOptionButton &opt)
{
	opt.init(this);
	if (isDown()) {
		opt.state |= QStyle::State_Selected;
	} else {
		opt.state &= ~QStyle::State_Selected;
	}
	if (decoration()->isActive()) {
		opt.state |= QStyle::State_Active;
	} else {
		opt.state &= ~QStyle::State_Active;
	}
}


void IaOraDecorationButton::paintEvent(QPaintEvent */* event */)
{
	const KCommonDecoration *deco = decoration();
	QPainter painter(this);
	QStyleOptionButton opt;
	initStyleOption(opt);

	// FIXME improve highlight
	QColor bgcolor = QColor(0, 0, 0, 0);
	if (opt.state & QStyle::State_MouseOver && !isDown()) {
		if (type() == CloseButton) {
			if (opt.state & QStyle::State_Active) {
				bgcolor = QColor(255, 100, 100, 120);
			} else {
				bgcolor = QColor(255, 100, 100, 70);
			}
		} else {
			if (opt.state & QStyle::State_Active) {
				bgcolor = QColor(255, 255, 255, 40);
			} else {
				bgcolor = QColor(255, 255, 255, 140);
			}
		}
	}
	painter.fillRect(rect().adjusted(1, 1, -1, -1), bgcolor);
	QStyle::StandardPixmap sp;
	switch (type()) {
		case OnAllDesktopsButton: sp = QStyle::StandardPixmap(QStyle::SP_CustomBase + (deco->isOnAllDesktops() ? 2 : 1)); break;
		case MenuButton:	sp = QStyle::StandardPixmap(QStyle::SP_CustomBase + 3); break;
		case AboveButton:	sp = QStyle::StandardPixmap(QStyle::SP_CustomBase + (deco->keepAbove() ? 6 : 4)); break;
		case BelowButton:	sp = QStyle::StandardPixmap(QStyle::SP_CustomBase + (deco->keepBelow() ? 7 : 5)); break;
		case HelpButton:	sp = QStyle::SP_TitleBarContextHelpButton; break;
		case MinButton:		sp = QStyle::SP_TitleBarMinButton; break;
		case MaxButton:	sp = deco->maximizeMode() == KDecoration::MaximizeFull ? QStyle::SP_TitleBarNormalButton : QStyle::SP_TitleBarMaxButton; break;
		case ShadeButton:	sp = deco->isShade() ? QStyle::SP_TitleBarUnshadeButton : QStyle::SP_TitleBarShadeButton; break;
		case CloseButton:	sp = QStyle::SP_TitleBarCloseButton; break;
		default:			sp = QStyle::SP_CustomBase; break;
	}

	QIcon icon;
	int shift;
	// paint shape
	opt.rect = rect().adjusted(1,1,-1,-1);
	opt.palette.setColor(QPalette::Text, deco->options()->color(KCommonDecoration::ColorFont, opt.state & QStyle::State_Active));
	icon = sp == QStyle::SP_CustomBase + 3 ? deco->icon().pixmap(16, 16) : style()->standardIcon(sp, &opt, this);
	shift = opt.state & QStyle::State_Selected ? 1 : 0;
	QPixmap pix = icon.pixmap(opt.rect.size());
	painter.setRenderHint(QPainter::Antialiasing, false);
	painter.drawPixmap(opt.rect.topLeft() + QPoint(shift, shift), pix);
	//qDebug("Rect: %d %d %dx%d", rect().left(), rect().top(), rect().width(), rect().height());
}


/*-----------------------------------------------------------------------*/

IaOraDecoration::IaOraDecoration(KDecorationBridge *bridge, KDecorationFactory *factory)
	: KCommonDecoration(bridge, factory)
{
	/* */
}


IaOraDecoration::~IaOraDecoration()
{
	/* */
}


QString IaOraDecoration::visibleName() const
{
	return i18n("IaOra");
}


void IaOraDecoration::initStyleOption(QStyleOption &opt)
{
	opt.init(widget());
	opt.fontMetrics = QFontMetrics(options()->font(isActive()));
	if (isActive()) {
		opt.state |= QStyle::State_Active;
	} else {
		opt.state &= ~QStyle::State_Active;
	}
}


bool IaOraDecoration::decorationBehaviour(DecorationBehaviour behaviour) const
{
	switch (behaviour) {
		case DB_MenuClose:
		case DB_WindowMask:
		case DB_ButtonHide:
			return true;
		default:
			return KCommonDecoration::decorationBehaviour(behaviour);
	}
}


void IaOraDecoration:: updateWindowShape()
{
	QStyleOptionTitleBar option;
	initStyleOption(option);
	QStyleHintReturnMask mask;
	if ((widget()->style())->styleHint(QStyle::SH_WindowFrame_Mask, &option, widget(), &mask)) {
		setMask(mask.region);
	} else {
		setMask(QRegion(widget()->rect()));
	}
}


int IaOraDecoration::layoutMetric(LayoutMetric lm, bool respectWindowState, const KCommonDecorationButton *button) const
{
	bool border = !(maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows());

	if (border) {
		switch (lm) {
			case LM_BorderLeft:
			case LM_BorderRight:
			case LM_BorderBottom:
				return borderWidth;
			case LM_TitleBorderLeft:
			case LM_TitleBorderRight:
				return 4;
			case LM_TitleEdgeLeft:
			case LM_TitleEdgeRight:
				return 8;
			case LM_TitleEdgeTop:
				return 2;
			case LM_ButtonHeight:
				return titleHeight;
			default:
				break;
		}
	} else {
		switch (lm) {
			case LM_BorderLeft:
			case LM_BorderRight:
			case LM_BorderBottom:
			case LM_TitleBorderLeft:
			case LM_TitleBorderRight:
			case LM_TitleEdgeLeft:
			case LM_TitleEdgeRight:
			case LM_TitleEdgeTop:
				return 2;
			case LM_ButtonHeight:
				return titleHeight;
			default:
				break;
		}
	}
	switch (lm) {
		case LM_TitleEdgeBottom:
			return 1;
		case LM_TitleHeight:
			return titleHeight + 2;
		case LM_ButtonWidth:
			return titleHeight;
		case LM_ButtonSpacing:
			return 0;
		case LM_ExplicitButtonSpacer:
			return 2;
		case LM_ButtonMarginTop:
			return 0;
		default:
			return KCommonDecoration::layoutMetric(lm, respectWindowState, button);
	}
}


KCommonDecorationButton *IaOraDecoration::createButton(ButtonType type)
{
	return new IaOraDecorationButton(type, this);
}


void IaOraDecoration::init()
{
	QWidget wid;
	setIaOraStyle(&wid);
	if (wid.style()) {
		QStyleOptionTitleBar option;
		option.init(&wid);
		option.fontMetrics = QFontMetrics(options()->font(true));
		borderWidth = wid.style()->pixelMetric(QStyle::PM_MdiSubWindowFrameWidth, &option, 0);
		titleHeight = wid.style()->pixelMetric(QStyle::PM_TitleBarHeight, &option, 0);
		noBorder = wid.style()->styleHint(QStyle::SH_TitleBar_NoBorder, &option, 0);
		autoRaise = wid.style()->styleHint(QStyle::SH_TitleBar_AutoRaise, &option, 0);
	} else {
		borderWidth = 4;
		titleHeight = 16;
		noBorder = false;
		autoRaise = false;
	}
	KConfig configFile(QString::fromAscii("kwiniaorarc"));
	KConfigGroup conf(&configFile, "General");
	coloredFrame = conf.readEntry("UseTitleBarBorderColors", false);
	contrastFrame = conf.readEntry("UseExtraContrastBorder", false);
	switch (options()->preferredBorderSize(factory())) {
		case BorderTiny:		borderWidth = 2; break;
		default:
		case BorderNormal:		borderWidth = 3; break;
		case BorderLarge:		borderWidth = 4; break;
		case BorderVeryLarge:	borderWidth = 5; break;
		case BorderHuge:		borderWidth = titleHeight / 2; break;
		case BorderVeryHuge:	borderWidth = titleHeight * 3 / 4; break;
		case BorderOversized:	borderWidth = titleHeight; break;
	}
	if (contrastFrame) {
		borderWidth += 1;
	}
	KCommonDecoration::init();
	if (wid.style()) widget()->setStyle(wid.style());
	widget()->setAutoFillBackground(false);
	widget()->setAttribute(Qt::WA_NoSystemBackground, true);
	widget()->setAttribute(Qt::WA_OpaquePaintEvent, true);
	widget()->setAttribute(Qt::WA_PaintOnScreen, false);
}


static inline QColor blend_color(const QColor &c0, const QColor &c1, qreal blend)
{
	int b = int(0.5 + 256.0 * blend);
	b = qMin(256, qMax(0, b));
	QRgb rgba0 = c0.rgba();
	QRgb rgba1 = c1.rgba();
	return QColor(
		qRed(rgba0) + (((qRed(rgba1) - qRed(rgba0)) * b) >> 8),
		qGreen(rgba0) + (((qGreen(rgba1) - qGreen(rgba0)) * b) >> 8),
		qBlue(rgba0) + (((qBlue(rgba1) - qBlue(rgba0)) * b) >> 8),
		qAlpha(rgba0) + (((qAlpha(rgba1) - qAlpha(rgba0)) * b) >> 8)
	);
}


void IaOraDecoration::paintEvent(QPaintEvent */*event */)
{
	bool border = !(maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows());
//	QWidget *w = widget();
	QStylePainter painter(widget());

	// draw the title bar
	QStyleOptionTitleBar option;
	initStyleOption(option);
	option.titleBarState = option.state;
	option.subControls = QStyle::SC_TitleBarLabel; // | QStyle::SC_TitleBarSysMenu;
	if (noBorder) {
		option.rect.setHeight(titleHeight);
	} else {
		option.rect.adjust(4, 4, -4, -4);
		option.rect.setHeight(titleHeight);
	}
	if (!border) {
		option.rect.adjust(-6, 0, 6, 0);
	}
	option.text = QString();
#if 0
	option.icon = decoration()->icon();
	option.titleBarFlags = Qt::WindowSystemMenuHint;
#else
	option.titleBarFlags = 0;
#endif
//	option.palette = options()->palette(ColorTitleBar, isActive());
	if (coloredFrame && border) {
		painter.fillRect(widget()->rect(), options()->color(ColorTitleBar, isActive()));
#if 0
		QColor color0 = option.palette.color(QPalette::Window);
		QColor color1 = options()->color(ColorTitleBar, isActive());
		QRect r = widget()->rect();
		for (int i = 0; i < borderWidth; ++i) {
			painter.setPen(blend_color(color1, color0, i / (borderWidth - 1.0)));
			painter.drawRect(r.adjusted(0, 0, -1, -1));
			r.adjust(1, 1, -1, -1);
		}
#endif
	} else {
		painter.fillRect(widget()->rect(), option.palette.color(QPalette::Window));
	}
	QFont font = options()->font(isActive());
//	font.setBold(false);
//	font.setItalic(true);
//	font.setPointSizeF(font.pointSizeF() / 1.19);
	painter.setFont(font);
	option.palette.setColor(QPalette::Highlight, options()->color(ColorTitleBar, true));
	option.palette.setColor(QPalette::HighlightedText, options()->color(ColorFont, true));
	option.palette.setColor(QPalette::Window, options()->color(ColorTitleBar, false));
	option.palette.setColor(QPalette::WindowText, options()->color(ColorFont, false));
//	option.rect = painter.style()->subControlRect(QStyle::CC_TitleBar, &option, QStyle::SC_TitleBarCloseButton, widget());
	painter.drawComplexControl(QStyle::CC_TitleBar, option);

	// draw the title caption
	// TODO use MDI code
	painter.save();
	qreal opacity = painter.opacity();
	int captionWidth = width() - buttonsLeftWidth() - buttonsRightWidth() - (border ? 16 : 4);
	option.text = option.fontMetrics.elidedText(caption(), Qt::ElideMiddle, captionWidth);
	QRect labelRect = QRect((border ? 8 : 2) + buttonsLeftWidth(), 0, captionWidth, titleHeight + 4);
	painter.setOpacity(opacity * 0.1);
	painter.setPen(Qt::black);
	painter.drawText(labelRect.adjusted(1, 1, 1, 1), Qt::AlignLeft | Qt::AlignVCenter | Qt::TextSingleLine, option.text);
	painter.setOpacity(option.state & QStyle::State_Active ? opacity : 0.7 * opacity);
	painter.setPen(options()->color(ColorFont, isActive()));
	painter.drawText(labelRect, Qt::AlignLeft | Qt::AlignVCenter | Qt::TextSingleLine, option.text);
	painter.restore();

	// draw the frame
	if (border) {
		QStyleOptionFrame frameOptions;
		initStyleOption(frameOptions);
		if (coloredFrame) {
			frameOptions.palette.setColor(QPalette::Window, options()->color(ColorTitleBar, isActive()));
		}
		frameOptions.lineWidth = borderWidth;
		if (contrastFrame) {
			QBrush outerLineFill = QColor(170, 170, 170);
		//	QBrush outerLineFill = Qt::red;

			painter.fillRect(frameOptions.rect.adjusted(0, 0, 0, -frameOptions.rect.height() + 1), outerLineFill);
			painter.fillRect(frameOptions.rect.adjusted(0, frameOptions.rect.height() - 1, 0, 0), outerLineFill);
			painter.fillRect(frameOptions.rect.adjusted(0, 0, -frameOptions.rect.width() + 1, 0), outerLineFill);
			painter.fillRect(frameOptions.rect.adjusted(frameOptions.rect.width() - 1, 0, 0, 0), outerLineFill);
			frameOptions.rect.adjust(1, 1, -1, -1);
		}
		painter.drawPrimitive(QStyle::PE_FrameWindow, frameOptions);
	}
}


/*-----------------------------------------------------------------------*/

IaOraDecorationFactory::IaOraDecorationFactory()
{
}


IaOraDecorationFactory::~IaOraDecorationFactory()
{
	/* */
}


KDecoration *IaOraDecorationFactory::createDecoration(KDecorationBridge *bridge)
{
	return (new IaOraDecoration(bridge, this))->decoration();
}


bool IaOraDecorationFactory::reset(unsigned long changed)
{
	resetDecorations(changed);
	return true;
}


QList<IaOraDecorationFactory::BorderSize> IaOraDecorationFactory::borderSizes() const
{
	return QList<BorderSize>()
			<< BorderTiny
			<< BorderNormal
			<< BorderLarge
			<< BorderVeryLarge
			<< BorderHuge
			<< BorderVeryHuge
			<< BorderOversized;
}


bool IaOraDecorationFactory::supports(Ability ability) const
{
	switch (ability)
	{
		case AbilityAnnounceButtons:
		case AbilityButtonMenu:
		case AbilityButtonHelp:
		case AbilityButtonShade:
		case AbilityButtonMinimize:
		case AbilityButtonMaximize:
		case AbilityButtonClose:
		case AbilityButtonSpacer:
		case AbilityButtonOnAllDesktops:
		case AbilityButtonAboveOthers:
		case AbilityButtonBelowOthers:
			return true;
		case AbilityAnnounceColors:
		case AbilityColorTitleBack:
		case AbilityColorTitleFore:
			return true;
		default:
			return false;
	}
}


/*-----------------------------------------------------------------------*/

#include "kwin_iaora.moc"


