/*
 * Copyright 2008 Gustavo Pichorim Boiko <boiko@mandriva.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "iaorastyle.h"

void IaOraStyle::drawSlider(const QStyleOption *opt, QPainter *p, 
                            const QRect &grooveRegion, const QRect &handle, const QRect &ticks)
{
	const QStyleOptionSlider *slider = qstyleoption_cast<const QStyleOptionSlider *>(opt);
	if (!slider)
		return;

	ColorData *data = lookupData(opt->palette);
	QPen oldPen = p->pen();
	
	bool active = (opt->state & QStyle::State_Enabled);
	bool vertical = slider->orientation == Qt::Vertical;
	bool ticksAbove = slider->tickPosition & QSlider::TicksAbove;
	bool ticksBelow = slider->tickPosition & QSlider::TicksBelow;
			
	
	QColor c(Qt::yellow);
	c.setAlpha(100);
			
	if (slider->subControls & QStyle::SC_SliderGroove)
	{
		QRect inside = grooveRegion.adjusted(1,1,-1,-1);
		p->fillRect(inside, data->gray[0]);
		drawFrameCuttingOne(p, grooveRegion, active ? data->gray[5] : data->gray[4]);
	}
	if (slider->subControls & QStyle::SC_SliderHandle)
	{
		QRect circle(0,0,4,4);
		circle.moveCenter(handle.center());
		QLinearGradient g1, g2;
		if (vertical)
		{
			g1 = QLinearGradient(handle.left(), 0, handle.right(), 0);
			g2 = QLinearGradient(circle.left(), 0, circle.right(), 0);
		}
		else
		{
			g1 = QLinearGradient(0, handle.top(), 0, handle.bottom()); 
			g2 = QLinearGradient(0, circle.top(), 0, circle.bottom());
		}

		g1.setColorAt(0., data->color[2]);
		g1.setColorAt(1., data->color[4]);
		g2.setColorAt(0., data->color[4]);
		g2.setColorAt(1., data->color[2]);
		
		fillRectCuttingOne(p, handle, g1);
	
		p->setPen(Qt::NoPen);
		p->setBrush(g2);
		p->drawEllipse(circle);

	}
	
	// TODO: do the ticks
	Q_UNUSED(ticksAbove);
	Q_UNUSED(ticksBelow);
	p->fillRect(ticks, c);
	
	p->setPen(oldPen);
}


