/*
 * Copyright 2008 Gustavo Pichorim Boiko <boiko@mandriva.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "iaorastyle.h"

void IaOraStyle::drawTabBarTab(const QStyleOption *opt, QPainter *p)
{
	const QStyleOptionTab *tab = qstyleoption_cast<const QStyleOptionTab *>(opt);
	QRect r = opt->rect;
	
	if (!tab)
		return;
	
	ColorData *data = lookupData(opt->palette);
	bool selected = (tab->state & QStyle::State_Selected);
	int top = r.top(), bottom = r.bottom(), left = r.left(), right = r.right();
	
	p->save();
	
	if (selected) // active tab
	{
		QRect contents = opt->rect.adjusted(1,1,-1,-1);
		QLine borders[2];
		QLine lightBlue[2];
		QLine darkBlue[2];
		
		if (tab->shape == QTabBar::RoundedNorth || tab->shape == QTabBar::TriangularNorth)
		{
			contents.adjust(0,3,0,1);
			borders[0] = QLine(left, top+4, left, bottom);
			borders[1] = QLine(right, top+4, right, bottom);
			lightBlue[0] = QLine(left+2,top,right-2,top);
			lightBlue[1] = QLine(left+1,top+1,right-1,top+1);
			darkBlue[0] = QLine(left,top+2,right,top+2);
			darkBlue[1] = QLine(left,top+3,right,top+3);
		}
		else if (tab->shape == QTabBar::RoundedSouth || tab->shape == QTabBar::TriangularSouth)
		{
			contents.adjust(0,-1,0,-3);
			borders[0] = QLine(left,top,left,bottom-4);
			borders[1] = QLine(right,top,right,bottom-4);
			lightBlue[0] = QLine(left, bottom-3, right, bottom-3);
			lightBlue[1] = QLine(left, bottom-2, right, bottom-2);
			darkBlue[0] = QLine(left+1, bottom-1, right-1, bottom-1);
			darkBlue[1] = QLine(left+2, bottom, right-2, bottom);
		}
		else if (tab->shape == QTabBar::RoundedEast || tab->shape == QTabBar::TriangularEast) // right
		{
			contents.adjust(-1,0,-3,0);
			borders[0] = QLine(left, top, right-4, top);
			borders[1] = QLine(left, bottom, right-4, bottom);
			lightBlue[0] = QLine(right-3, top, right-3, bottom);
			lightBlue[1] = QLine(right-2, top, right-2, bottom);
			darkBlue[0] = QLine(right-1, top+1, right-1, bottom-1);
			darkBlue[1] = QLine(right, top+2, right, bottom-2);
		}
		else // left
		{
			contents.adjust(3,0,1,0);
			borders[0] = QLine(left+4, top, right, top);
			borders[1] = QLine(left+4, bottom, right, bottom);
			lightBlue[0] = QLine(left, top+2, left, bottom-2);
			lightBlue[1] = QLine(left+1, top+1, left+1, bottom-1);
			darkBlue[0] = QLine(left+2, top, left+2, bottom);
			darkBlue[1] = QLine(left+3, top, left+3, bottom);
		}
		
		// fill the contents
		p->fillRect(contents, data->gray[0]);
		
		// draw the gray borders
		p->setPen(data->gray[4]);
		p->drawLines(borders, 2);
		
		// the blue lines
		p->setPen(data->color[2]);
		p->drawLines(lightBlue, 2);
		p->setPen(data->color[4]);
		p->drawLines(darkBlue, 2);
	}
	else // inactive tab
	{
		QRect contents = opt->rect.adjusted(1,1,-1,-1);
		
		QPoint points[2];
		QLine lines[4];
		if (tab->shape == QTabBar::RoundedNorth || tab->shape == QTabBar::TriangularNorth)
		{
			top+=3;
			contents.setTop(top+1);
			lines[0] = QLine(left+2,top,right-2,top); // top
			lines[1] = QLine(left, top+2, left, bottom); // left
			lines[2] = QLine(right, top+2, right, bottom); // right
			lines[3] = QLine(left, bottom, right, bottom); // bottom
			points[0] = QPoint(left+1, top+1);
			points[1] = QPoint(right-1, top+1);
		}
		else if (tab->shape == QTabBar::RoundedSouth || tab->shape == QTabBar::TriangularSouth)
		{
			bottom-=3;
			contents.setBottom(bottom-1);
			lines[0] = QLine(left,top,right,top); // top
			lines[1] = QLine(left, top, left, bottom-2); // left
			lines[2] = QLine(right, top, right, bottom-2); // right
			lines[3] = QLine(left+2, bottom, right-2, bottom); // bottom
			points[0] = QPoint(left+1, bottom-1);
			points[1] = QPoint(right-1, bottom-1); 
		}
		else if (tab->shape == QTabBar::RoundedEast || tab->shape == QTabBar::TriangularEast) // right
		{
			right-=3;
			contents.setRight(right-1);
			lines[0] = QLine(left,top,right-2,top); // top
			lines[1] = QLine(left,top,left,bottom); // left
			lines[2] = QLine(right,top+2,right,bottom-2); // right
			lines[3] = QLine(left,bottom,right-2,bottom); // bottom
			points[0] = QPoint(right-1,top+1);
			points[1] = QPoint(right-1,bottom-1);
		}
		else // left
		{
			left+=3;
			contents.setLeft(left+1);
			lines[0] = QLine(left+2,top,right,top); // top
			lines[1] = QLine(left,top+1,left,bottom-2); // left
			lines[2] = QLine(right,top,right,bottom); // right
			lines[3] = QLine(left+2,bottom,right,bottom); // bottom
			points[0] = QPoint(left+1,top+1);
			points[1] = QPoint(left+1,bottom-1);
		}
		
		// draw the contents
		p->setPen(Qt::NoPen);
		p->setBrush(data->gray[1]);
		p->drawRect(contents);
		
		// draw the lines
		p->setPen(data->gray[5]);
		p->drawLines(lines, 4);
		p->drawPoints(points, 2);
	}

	p->restore();
}



